#=======================================================================
#	$Id: CGI.pm,v 1.2 2005/09/07 09:58:25 pythontech Exp $
#	Repository of web client session data
#	Copyright (C) 2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::SessionDB::CGI;
use strict;
use CGI::Session;

sub new {
    my $class = shift;
    my %conf = @_;
    my $self = {
	sessiondir => $conf{'sessiondir'} || '/tmp',
    };
    bless $self, $class;
    return $self;
}

#-----------------------------------------------------------------------
#	Find (or, if necessary, establish) a session for the given
#	query
#-----------------------------------------------------------------------
sub find_session {
    my($self, $query) = @_;
    my $cgisess = PythonTech::SessionDB::CGI::Session->new($query,
							   $self->{'sessiondir'});
    return $cgisess;
}

#-----------------------------------------------------------------------
#	Clean up expired sessions
#-----------------------------------------------------------------------
sub purge {
    # FIXME unimplemented
}

#=======================================================================
#	Session object
#=======================================================================
package PythonTech::SessionDB::CGI::Session;
use strict;
use CGI::Session;

sub new {
    my($class, $query,$dir) = @_;
    my $claimed_id = $query->cookie('CGISESSID');
    my $cgisess = CGI::Session->new(undef, $query,
				    {Directory => $dir});
    my $sessid = $cgisess->id;
#    print STDERR "claimed=$claimed_id actual=$sessid dir=$dir\n";
    if ($sessid ne $claimed_id) {
	$query->response->set_cookie(-name => 'CGISESSID',
				     -value => $sessid);
    }
    my $self = {
	server => $query->server,
	cgisess => $cgisess,
    };
    bless $self, $class;
    return $self;
}

sub id {
    my($self) = @_;
    return $self->{'cgisess'}->id;
}

sub flush {
    my($self) = @_;
    $self->{'cgisess'}->flush;
}

sub clear {
    my $self = shift;
    $self->{'cgisess'}->clear(@_);
}

#-----------------------------------------------------------------------
#	Get or set a session property
#-----------------------------------------------------------------------
sub get {
    my($self, $prop) = @_;
    return $self->{'cgisess'}->param($prop);
}

sub set {
    my($self, $prop,$value) = @_;
    $self->{'cgisess'}->param($prop, $value);
}

sub param {
    my $self = shift;
    return $self->{'cgisess'}->param(@_);
}

#-----------------------------------------------------------------------
#	Record that the user is now logged in.
#	Typically called from the 'login' script, though may be also
#	from a registration page etc.
#-----------------------------------------------------------------------
sub login {
    my($self, $username) = @_;
    $self->set('userName' => $username);
    $self->set('loggedIn' => 1);
}

#-----------------------------------------------------------------------
#	Check that the user is logged in and, if not, redirect them to
#	a page where they can authenticate themselves.
#-----------------------------------------------------------------------
sub require_login {
    my($self, $afterpage) = @_;
    print STDERR "deprecated SessionDB::CGI require_login\n";
    if (! $self->get('loggedIn')) {
	my $myurl = defined($afterpage) ? $afterpage :
	  $::PythonTech::CGI::request->self_url;
	my $loginurl = $self->{'server'}->get('LOGINURL');
	die "No login URL\n" unless $loginurl;
	$self->set('afterLogin' => $myurl);
	$::PythonTech::CGI::response->redirect($loginurl);
	$self->flush;
	exit 0;
    }
}

1;
